@@ -5,7 +5,6 @@ module Network.Wai.Middleware.Autohead (autohead) where
 
 import Network.Wai
 import Data.Monoid (mempty)
-import Data.Enumerator (enumEOF, ($$))
 
 autohead :: Middleware
 autohead app req
@@ -14,10 +13,6 @@ autohead app req
         case res of
             ResponseFile s hs _ _ -> return $ ResponseBuilder s hs mempty
             ResponseBuilder s hs _ -> return $ ResponseBuilder s hs mempty
-            ResponseEnumerator e -> do
-                let helper f =
-                        let helper' s hs = enumEOF $$ f s hs
-                         in e helper'
-                return $ ResponseEnumerator helper
+            ResponseSource s hs _ -> return $ ResponseBuilder s hs mempty
     | otherwise = app req
 
