@@ -21,6 +21,7 @@ import Network.Wai.Middleware.Gzip
 import Network.Wai.Middleware.Vhost
 import Network.Wai.Middleware.Autohead
 import Network.Wai.Middleware.MethodOverride
+import Network.Wai.Middleware.MethodOverridePost
 import Network.Wai.Middleware.AcceptOverride
 import Network.Wai.Middleware.RequestLogger (logCallback)
 import Codec.Compression.GZip (decompress)
@@ -31,7 +32,6 @@ import Data.Conduit.Binary (sourceFile)
 import Control.Monad.IO.Class (liftIO)
 import Data.Maybe (fromMaybe)
 import Network.HTTP.Types (parseSimpleQuery, status200)
-import Data.Monoid (mappend)
 
 specs :: Specs
 specs = do
@@ -53,6 +53,7 @@ specs = do
     it "vhost" caseVhost
     it "autohead" caseAutohead
     it "method override" caseMethodOverride
+    it "method override post" caseMethodOverridePost
     it "accept override" caseAcceptOverride
     it "dalvik multipart" caseDalvikMultipart
     it "debug request body" caseDebugRequestBody
@@ -354,6 +355,32 @@ caseMethodOverride = flip runSession moApp $ do
                 }
     assertHeader "Method" "PUT" sres3
 
+mopApp :: Application
+mopApp = methodOverridePost $ \req -> return $ responseLBS status200 [("Method", requestMethod req)] ""
+
+caseMethodOverridePost :: Assertion
+caseMethodOverridePost = flip runSession mopApp $ do
+
+    -- Get Request are unmodified
+    sres1 <- let r = toRequest "application/x-www-form-urlencoded" "_method=PUT&foo=bar&baz=bin"
+                 s = simpleRequest r
+                 m = s { requestMethod = "GET" }
+                 b = r { simpleRequest = m }
+             in srequest b
+    assertHeader "Method" "GET" sres1
+
+    -- Post requests are modified if _method comes first
+    sres2 <- srequest $ toRequest "application/x-www-form-urlencoded" "_method=PUT&foo=bar&baz=bin"
+    assertHeader "Method" "PUT" sres2
+
+    -- Post requests are unmodified if _method doesn't come first
+    sres3 <- srequest $ toRequest "application/x-www-form-urlencoded" "foo=bar&_method=PUT&baz=bin"
+    assertHeader "Method" "POST" sres3
+
+    -- Post requests are unmodified if Content-Type header isn't set to "application/x-www-form-urlencoded"
+    sres4 <- srequest $ toRequest "text/html; charset=utf-8" "foo=bar&_method=PUT&baz=bin"
+    assertHeader "Method" "POST" sres4
+
 aoApp :: Application
 aoApp = acceptOverride $ \req -> return $ responseLBS status200
     [("Accept", fromMaybe "" $ lookup "Accept" $ requestHeaders req)] ""
@@ -430,9 +457,9 @@ caseDebugRequestBody = do
   where
     params = [("foo", "bar"), ("baz", "bin")]
     -- FIXME change back once we include post parameter output in logging postOutput = T.pack $ "POST \nAccept: \nPOST " ++ (show params)
-    postOutput = T.pack $ "POST / Accept: \n"
+    postOutput = T.pack $ "POST / Accept: \nStatus: 200 OK"
     -- FIXME getOutput _qs = T.pack $ "GET /location" ++ "\nAccept: \nGET " ++ (show params) -- \nAccept: \n" ++ (show params)
-    getOutput _qs = T.pack $ "GET /location?foo=bar&baz=bin Accept: \n"
+    getOutput _qs = T.pack $ "GET /location?foo=bar&baz=bin Accept: \nStatus: 200 OK"
 
     debugApp output' = logCallback (\t -> liftIO $ assertEqual "debug" output t) $ \_req -> do
         return $ responseLBS status200 [ ] ""
